c
c---------------  tens3 = 3point tensors ------------------------------
c
      subroutine tens3( mt, p1sq, p2sq, psq, B0, C0, 
     &                  Cij)
      implicit none
      double precision mt, p1sq, p2sq, psq
      complex*16 B0(3), C0, Cij(2,4)
c
c  determine the Passarino-Veltman tensor decomposition for the 
c  three-point tensor integrals
c
c                   1                 1;  k_mu;   k_mu k_nu
c C0;C_mu;C_munu =------Int d^4k --------------------------------------
c                 i pi^2        [-k^2+mt^2][-(k+p1)^2+mt^2][-(k+p1+p2)^2+mt^2] 
c
c with
c
c   C_mu = p1_mu C11  +  p_2_mu C12
c
c  C_munu = p1_mu p1_nu C21 + p2_mu p2_nu C22 + 
c           (p1_mu p2_nu + p1_nu p2_mu) C23  -  g_munu C24
c
c  for notation see Passarino&Veltman, NP B160 (1979) 151 and my notes
c  on gg-->Hgg, pages 4.1--4.8
c
C INPUT:  mt, p1sq, p2sq, psq=(p1+p2)^2     mass and 4-momenta**2
C         B0, C0                   4 scalar integrals; the 3 B0 are, 
c                                  in PV notation:
c         B0(1) = B0(1,2)          B_0 function with subtraction of 
c         B0(2) = B0(2,3)          divergent term
c         B0(3) = B0(1,3)
c
c OUTPUT: Cij(n,m) = C_nm          form factors in the tensor integrals
c         n=1,2; m=1,2,3,4         a la PV
c
c	Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c	Initial version:  2000 November
c	Last modified:    

      real*8 f1, f2, Xi(2,2), p1p2, deti
      complex*16 b13, b12, b23, r1,r2,r3,r4,r5,r6
      complex*16 C11, C12, C21, C22, C23, C24, C23p
      integer i,j, iscount
      logical ldebug
      parameter (ldebug=.false.)
      data iscount /0/
      save iscount

      real*8 srt
      external srt

      p1p2 = (psq-p1sq-p2sq)/2
      f1 = p1sq
      f2 = psq - p1sq
      if (psq.eq.0d0) then
         deti = 0.25d0*(p1sq-p2sq)**2
      elseif (p1sq.eq.0d0) then
         deti = 0.25d0*(psq-p2sq)**2
      elseif (p2sq.eq.0d0) then
         deti = 0.25d0*(psq-p1sq)**2
      else
         deti = p1p2**2-p1sq*p2sq
         if (abs(deti).lt.1d-30) then
            do i=1,2
               do j = 1,4
                  Cij(i,j) = 0
               enddo
            enddo
            iscount = iscount+1
            if (iscount.le.1000) then
               print*,' Warning: singular point in TENS3 '
               return
            else
               print*,' stop: too many singular points in TENS3 '
               stop
            endif
         endif
         if (abs(deti).lt.1d-6*max(abs(p1p2**2),abs(p1sq*p2sq))) then
            print*,' singular Cij. deti suppressed by ',
     1           abs(deti)/max(abs(p1p2**2),abs(p1sq*p2sq))
            print*,' p1 =  ',srt(p1sq)
            print*,' p2 =  ',srt(p2sq)
            print*,' p12 = ',srt(psq)
         endif         
      endif
      deti = 1/deti
      Xi(1,1) = p2sq*deti
      Xi(1,2) = -p1p2*deti
      Xi(2,1) = -p1p2*deti
      Xi(2,2) = p1sq*deti
      b12 = B0(1)
      b23 = B0(2)
      b13 = B0(3)
c
      R1 = (f1*C0+b13-b23)/2
      R2 = (f2*C0+b12-b13)/2
      C11 = Xi(1,1)*R1 + Xi(1,2)*R2
      C12 = Xi(2,1)*R1 + Xi(2,2)*R2
      Cij(1,1) = C11
      Cij(1,2) = C12
      Cij(1,3) = 0
      Cij(1,4) = 0
c
      C24 = -0.5d0*mt**2*C0 + 0.25d0*(1+b23-f1*C11-f2*C12)
      R3 = -b13/4 + (b23+f1*C11)/2 - C24
      R4 = -(b13-b23)/4 + f1*C12/2
      R5 = -(b12-b13)/4 + f2*C11/2
      R6 = b13/4 + f2*C12/2 - C24

      Cij(2,1) = Xi(1,1)*R3 + Xi(1,2)*R5
      Cij(2,3) = Xi(2,1)*R3 + Xi(2,2)*R5
      Cij(2,2) = Xi(2,1)*R4 + Xi(2,2)*R6
      Cij(2,4) = C24

      if (ldebug) then
         c21 = cij(2,1)
         c22 = cij(2,2)
         c23 = cij(2,3)
         C23p= Xi(1,1)*R4 + Xi(1,2)*R6
         print*,' C23 comparison ',C23, C23p,C23/(C23p+1d-30)
         print*,'r1:',p1sq*C11+p1p2*c12+0.5d0*(p1sq*c0+b13-b23)
         print*,'r2:',p1p2*C11+p2sq*C12+0.5d0*((p2sq+2*p1p2)*c0+
     &                b12-b13)
c         print*,' c24 = ',c24
c         r1 = p1sq*C21+p2sq*C22+2*p1p2*C23-4*c24+b23-mt**2*C0
c         print*,r1,r1/(c24+1d-60)
         print*,'r4f',p1p2*c22+p1sq*c23+(b13-b23)/4+p1sq*c12/2
         print*,'r4:',p1p2*c22+p1sq*c23-(b13-b23)/4+p1sq*c12/2
         print*,'r5f',p1p2*c21+p2sq*c23+(b12-b13)/4+(2*p1p2+p2sq)*c11/2
         print*,'r5:',p1p2*c21+p2sq*c23-(b12-b13)/4+(2*p1p2+p2sq)*c11/2
      endif

      return
      end

c
c---------------  tens4 = 4point tensors -----------------------------
c
      subroutine tens4( mt, p1sq, p2sq, p3sq, p1p2, p1p3, p2p3, 
     &                  C0123,  C0124,  C0134,  C0234,
     &                  Cij123, Cij124, Cij134, Cij234, 
     &                  D0, Dij)
      implicit none
      double precision mt, p1sq, p2sq, p3sq, p1p2, p1p3, p2p3
      complex*16 C0123,  C0124,  C0134,  C0234
      complex*16 Cij123(2,4), Cij124(2,4), Cij134(2,4), Cij234(2,4)
      complex*16 D0, Dij(3,13)
c
c  determine the Passarino-Veltman tensor decomposition for the 
c  four-point tensor integrals
c
c                                       1
c   D0; D_mu; D_mu,nu; D_mu,nu,rho = ------- Int d^4k
c                                     i pi^2 
c
c           1;  k_mu;   k_mu k_nu; k_mu k_nu k_rho
c   -------------------------------------------------------------------
c   [-k^2+mt^2][-(k+p1)^2+mt^2][-(k+p1+p2)^2+mt^2][-(k+p1+p2+p3)^2+m^2]
c
c  with
c
c   D_mu = p1_mu D11  +  p2_mu D12  +  p3_mu D13
c
c   D_munu = p1_mu p1_nu D21 + p2_mu p2_nu D22 + ...
c
c  for notation see Passarino&Veltman, NP B160 (1979) 151 and my notes
c  on gg-->Hgg, pages 4.9--4.19. Note: since I use Bjorken Drell g_munu
c  one needs to replace pi.pj --> -pi.pj, delta_munu = -g_munu in PV
c  to get the results below
c
C INPUT:  mt, p1sq, p1p2,...                mass and 4-momenta**2
C         C0123 = C0(1,2,3) = C0(p1,p2)     scalar three point 
C         C0124 = C0(1,2,4) = C0(p1,p2+p3)  functions in PV notation
C         C0134 = C0(1,3,4) = C0(p1+p2,p3)
C         C0234 = C0(2,3,4) = C0(p2,p3)
C         Cij123(n,m) = C_nm(1,2,3) ....    higher C_nm form factors
C                                           as in tens3
c         D0 = D0(p1,p2,p3)                 scalar four point function
c
c OUTPUT: Dij(n,m) = D_nm                   form factors in the tensor 
c                                           integrals a la PV
c         nm = 11, 12, 13                   ff's for D_mu
c         nm = 21, 22, 23, 24, 25, 26, 27   ff's for D_munu
c         nm = 31, 32, 33, ..., 39, 310, 311, 312  ff's for D_mu,nu,rho
c
c	Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c	Initial version:  2000 December 
c	Last modified:    2002 October 29

      real*8 f1, f2, f3, Xi(3,3), X(3,3), deti
      complex*16 r(20:58), Dijp(3,13), t(9), d310pp
      integer i,j, iscount
      logical ldebug, ldb0
      parameter (ldebug=.false.)
      common /bdebug/ ldb0
      real*8 srt !, dtexm
      external srt
      data iscount /0/
      save iscount

      f1 = p1sq
      f2 = p2sq+2*p1p2
      f3 = p3sq+2*(p1p3+p2p3)
      deti = -p1sq*p2sq*p3sq-2*p1p2*p2p3*p1p3
     &       +p1sq*p2p3**2+p2sq*p1p3**2+p3sq*p1p2**2
      do i=1,3
         do j = 1,13
            Dij(i,j) = 0
         enddo
      enddo
      if (abs(deti).lt.1d-30) then
         iscount = iscount+1
         if (iscount.le.1000) then
            print*,' Warning: singular point in TENS4 '
            return
         else
            print*,' stop: too many singular points in TENS4 '
            stop
         endif
      endif
      deti = 1/deti
c
      Xi(1,1) = (p2sq*p3sq-p2p3**2)*deti
      Xi(1,2) = (p1p3*p2p3-p3sq*p1p2)*deti
      Xi(1,3) = (p1p2*p2p3-p2sq*p1p3)*deti

      Xi(2,1) = Xi(1,2)
      Xi(2,2) = (p1sq*p3sq-p1p3**2)*deti
      Xi(2,3) = (p1p2*p1p3-p1sq*p2p3)*deti

      Xi(3,1) = Xi(1,3)
      Xi(3,2) = Xi(2,3)
      Xi(3,3) = (p1sq*p2sq-p1p2**2)*deti

      if (ldb0) then
         print('(g15.4)'),' det(-X) = ',1/deti
         print*,' 1,2,3 ',srt(p1sq),srt(p2sq),srt(p3sq)
c         if (p1sq.ne.0) then
c            dtexm = (p1p2+p2p3)**2*(p1p3+p2p3)**2/
c     1              (p1sq+p2sq+p3sq+2*(p1p2+p1p3+p2p3))
c         elseif (p2sq.ne.0) then
c            dtexm = (p1p2+p1p3)**2*(p1p3+p2p3)**2/
c     1              (p1sq+p2sq+p3sq+2*(p1p2+p1p3+p2p3))
c         else
c            dtexm = (p1p2+p1p3)**2*(p1p2+p2p3)**2/
c     1              (p1sq+p2sq+p3sq+2*(p1p2+p1p3+p2p3))
c         endif
c         print*,' detx ratio ',1/deti/dtexm
         print*,' 12 ',srt(p1sq+2*p1p2+p2sq),' 13 ',
     1          srt(p1sq+2*p1p3+p3sq),' 23 ',srt(p2sq+2*p2p3+p3sq)
         print*,' check inversion of X matrix for n = 3 '
         X(1,1) = -p1sq
         X(1,2) = -p1p2
         X(1,3) = -p1p3
         X(2,1) = -p1p2
         X(2,2) = -p2sq
         X(2,3) = -p2p3
         X(3,1) = -p1p3
         X(3,2) = -p2p3
         X(3,3) = -p3sq
         do i = 1,3
            do j = 1,3
               print*,'X*X^-1(',i,j,') = ', 
     &                X(i,1)*Xi(1,j)+X(i,2)*Xi(2,j)+X(i,3)*Xi(3,j)
            enddo
         enddo
         print*,' Xi = '
         do i = 1,3
            print 13,(Xi(i,j),j=1,3)
         enddo
         print*,' X = '
         do i = 1,3
            print 13,(X(i,j),j=1,3)
         enddo
 13      format (3g14.4)
      endif
c
      if (ldb0) then
         if (.false.) then
            C0123 = 0
            c0124 = 0
            c0234 = 0
            c0134 = 0
            do i=1,2
               do j = 1,4
                  cij123(i,j) = 0
                  cij124(i,j) = 0
                  cij234(i,j) = 0
                  cij134(i,j) = 0
               enddo
            enddo
         endif
         print*,' input for D1j '
         print*,' D0 = ',d0
         print*,' C0123 = ',C0123
         print*,' C0124 = ',C0124
         print*,' C0234 = ',C0234
         print*,' C0134 = ',C0134
      endif
      R(20) = (f1*D0+C0134-C0234)/2
      R(21) = (f2*D0+C0124-C0134)/2
      R(22) = (f3*D0+C0123-C0124)/2
      Dij(1,1) = Xi(1,1)*R(20) + Xi(1,2)*R(21) + Xi(1,3)*R(22)
      Dij(1,2) = Xi(2,1)*R(20) + Xi(2,2)*R(21) + Xi(2,3)*R(22)
      Dij(1,3) = Xi(3,1)*R(20) + Xi(3,2)*R(21) + Xi(3,3)*R(22)
c      do j = 4,13
c         Dij(1,j) = 0
c      enddo
ccccccc
      if (ldebug) then
         print*,' r20 = ',R(20)
         print*,' r21 = ',R(21)
         print*,' r22 = ',R(22)
         print*,'r20c ',
     &           -(p1sq*dij(1,1)+p1p2*dij(1,2)+p1p3*dij(1,3))/R(20)
         print*,'r21c ',
     &           -(p1p2*dij(1,1)+p2sq*dij(1,2)+p2p3*dij(1,3))/R(21)
         print*,'r22c ',
     &           -(p1p3*dij(1,1)+p2p3*dij(1,2)+p3sq*dij(1,3))/R(22)
      endif
ccccccc
      Dij(2,7) = -mt**2*D0 
     &           - (f1*dij(1,1)+f2*dij(1,2)+f3*dij(1,3)-C0234)/2
c
      R(30) = (f1*Dij(1,1)+Cij134(1,1)+C0234)/2 - Dij(2,7)
      R(33) = (f1*Dij(1,2)+Cij134(1,1)-Cij234(1,1))/2
      R(36) = (f1*Dij(1,3)+Cij134(1,2)-Cij234(1,2))/2

      R(31) = (f2*Dij(1,1)+Cij124(1,1)-Cij134(1,1))/2
      R(34) = (f2*Dij(1,2)+Cij124(1,2)-Cij134(1,1))/2 - Dij(2,7)
      R(37) = (f2*Dij(1,3)+Cij124(1,2)-Cij134(1,2))/2

      R(32) = (f3*Dij(1,1)+Cij123(1,1)-Cij124(1,1))/2
      R(35) = (f3*Dij(1,2)+Cij123(1,2)-Cij124(1,2))/2
      R(38) = (f3*Dij(1,3)-Cij124(1,2))/2 - Dij(2,7)

      Dij(2,1) = Xi(1,1)*R(30) + Xi(1,2)*R(31) + Xi(1,3)*R(32)
      Dij(2,4) = Xi(2,1)*R(30) + Xi(2,2)*R(31) + Xi(2,3)*R(32)
      Dij(2,5) = Xi(3,1)*R(30) + Xi(3,2)*R(31) + Xi(3,3)*R(32)

      Dij(2,2) = Xi(2,1)*R(33) + Xi(2,2)*R(34) + Xi(2,3)*R(35)
      Dij(2,6) = Xi(3,1)*R(33) + Xi(3,2)*R(34) + Xi(3,3)*R(35)

      Dij(2,3) = Xi(3,1)*R(36) + Xi(3,2)*R(37) + Xi(3,3)*R(38)
cccccccccc
      if (ldebug) then
         Dijp(2,4) =Xi(1,1)*R(33) + Xi(1,2)*R(34) + Xi(1,3)*R(35)
         Dijp(2,5) = Xi(1,1)*R(36) + Xi(1,2)*R(37) + Xi(1,3)*R(38)
         Dijp(2,6) = Xi(2,1)*R(36) + Xi(2,2)*R(37) + Xi(2,3)*R(38)
         print*,' ratios of 2 versions for D24, D25, D26 '
         print*,' D24 ',dij(2,4),dij(2,4)/dijp(2,4)
         print*,' D25 ',dij(2,5),dij(2,5)/dijp(2,5)
         print*,' D26 ',dij(2,6),dij(2,6)/dijp(2,6)
      endif
cccccccccc
c      do j = 8,13
c         Dij(2,j) = 0
c      enddo
c
      Dij(3,11) = -mt**2/2*dij(1,1) 
     &            - (f1*Dij(2,1)+f2*dij(2,4)+f3*dij(2,5)+C0234)/4
      Dij(3,12) = -mt**2/2*dij(1,2) 
     &            - (f1*Dij(2,4)+f2*dij(2,2)+f3*dij(2,6)-Cij234(1,1))/4
      Dij(3,13) = -mt**2/2*dij(1,3) 
     &            - (f1*Dij(2,5)+f2*dij(2,6)+f3*dij(2,3)-Cij234(1,2))/4
c
      R(41) = (f1*Dij(2,1)+Cij134(2,1)-C0234)/2 - 2*Dij(3,11)
      R(42) = (f2*Dij(2,1)+Cij124(2,1)-Cij134(2,1))/2
      R(43) = (f3*Dij(2,1)+Cij123(2,1)-Cij124(2,1))/2

      R(44) = (f1*Dij(2,4)+Cij134(2,1)+Cij234(1,1))/2 - Dij(3,12)
      R(50) = (f1*Dij(2,2)+Cij134(2,1)-Cij234(2,1))/2 
      R(56) = (f1*Dij(2,3)+Cij134(2,2)-Cij234(2,2))/2

      R(45) = (f2*Dij(2,4)+Cij124(2,3)-Cij134(2,1))/2 - Dij(3,11)
      R(51) = (f2*Dij(2,2)+Cij124(2,2)-Cij134(2,1))/2 - 2*Dij(3,12)
      R(57) = (f2*Dij(2,3)+Cij124(2,2)-Cij134(2,2))/2 

      R(46) = (f3*Dij(2,4)+Cij123(2,3)-Cij124(2,3))/2
      R(52) = (f3*Dij(2,2)+Cij123(2,2)-Cij124(2,2))/2
      R(58) = (f3*Dij(2,3)            -Cij124(2,2))/2 - 2*Dij(3,13) 

      Dij(3,1) = Xi(1,1)*R(41) + Xi(1,2)*R(42) + Xi(1,3)*R(43)
      Dij(3,4) = Xi(2,1)*R(41) + Xi(2,2)*R(42) + Xi(2,3)*R(43)
      Dij(3,5) = Xi(3,1)*R(41) + Xi(3,2)*R(42) + Xi(3,3)*R(43)

      Dij(3,6) = Xi(1,1)*R(50) + Xi(1,2)*R(51) + Xi(1,3)*R(52)
      Dij(3,2) = Xi(2,1)*R(50) + Xi(2,2)*R(51) + Xi(2,3)*R(52)
      Dij(3,8) = Xi(3,1)*R(50) + Xi(3,2)*R(51) + Xi(3,3)*R(52)

      Dij(3,7) = Xi(1,1)*R(56) + Xi(1,2)*R(57) + Xi(1,3)*R(58)
      Dij(3,9) = Xi(2,1)*R(56) + Xi(2,2)*R(57) + Xi(2,3)*R(58)
      Dij(3,3) = Xi(3,1)*R(56) + Xi(3,2)*R(57) + Xi(3,3)*R(58)

      Dij(3,10) = Xi(3,1)*R(44) + Xi(3,2)*R(45) + Xi(3,3)*R(46)
ccccccccccc
      if (ldb0) then
         print*,' R_20/R_22 = ',r(20)/r(22)
         print*,' R_30/R_32 = ',r(30)/r(32)
         print*,' R_33/R_35 = ',r(33)/r(35)
         print*,' R_36/R_38 = ',r(36)/r(38)

         print*,' R_41/R_43 = ',r(41)/r(43)
         print*,' R_50/R_52 = ',r(50)/r(52)
         print*,' R_56/R_58 = ',r(56)/r(58)
         print*,' R_44/R_46 = ',r(44)/r(46)
      endif
      if (ldebug) then
         Dijp(3,4) = Xi(1,1)*R(44) + Xi(1,2)*R(45) + Xi(1,3)*R(46)
         Dijp(3,6) = Xi(2,1)*R(44) + Xi(2,2)*R(45) + Xi(2,3)*R(46)
         print*,' ratios of 2 versions for D3j '
         print*,' D34 ',dij(3,4),dij(3,4)/dijp(3,4)
         print*,' D36 ',dij(3,6),dij(3,6)/dijp(3,6)
         t(1) = (cij134(2,3)+f1*Dij(2,5)+cij234(1,2))/2-dij(3,13)
         t(2) = (cij134(2,3)+f1*Dij(2,6)-cij234(2,3))/2
         t(3) = (cij134(2,4)+f1*Dij(2,7)-cij234(2,4))/2

         t(4) = (cij124(2,3)-cij134(2,3)+f2*Dij(2,5))/2
         t(5) = (cij124(2,2)-cij134(2,3)+f2*Dij(2,6))/2-dij(3,13)
         t(6) = (cij124(2,4)-cij134(2,4)+f2*Dij(2,7))/2

         t(7) = (           -cij124(2,3)+f3*Dij(2,5))/2-dij(3,11)
         t(8) = (           -cij124(2,2)+f3*Dij(2,6))/2-dij(3,12)
         t(9) = (cij123(2,4)-cij124(2,4)+f3*Dij(2,7))/2

         Dijp(3,5) = Xi(1,1)*t(1) + Xi(1,2)*t(4) + Xi(1,3)*t(7)
         Dijp(3,10)= Xi(2,1)*t(1) + Xi(2,2)*t(4) + Xi(2,3)*t(7)
         Dijp(3,7) = Xi(3,1)*t(1) + Xi(3,2)*t(4) + Xi(3,3)*t(7)

         D310pp    = Xi(1,1)*t(2) + Xi(1,2)*t(5) + Xi(1,3)*t(8)
         Dijp(3,8) = Xi(2,1)*t(2) + Xi(2,2)*t(5) + Xi(2,3)*t(8)
         Dijp(3,9) = Xi(3,1)*t(2) + Xi(3,2)*t(5) + Xi(3,3)*t(8)
         
         Dijp(3,11)= Xi(1,1)*t(3) + Xi(1,2)*t(6) + Xi(1,3)*t(9)
         Dijp(3,12)= Xi(2,1)*t(3) + Xi(2,2)*t(6) + Xi(2,3)*t(9)
         Dijp(3,13)= Xi(3,1)*t(3) + Xi(3,2)*t(6) + Xi(3,3)*t(9)

         print*,' D35 ',dij(3,5),dij(3,5)/dijp(3,5)
         print*,' D37 ',dij(3,7),dij(3,7)/dijp(3,7)
         print*,' D38 ',dij(3,8),dij(3,8)/dijp(3,8)
         print*,' D39 ',dij(3,9),dij(3,9)/dijp(3,9)
         print*,' D310 ',dij(3,10),dij(3,10)/dijp(3,10)
         print*,' D310pp ',d310pp,d310pp/dij(3,10)
         print*,' D311 ',dij(3,11),dij(3,11)/dijp(3,11)
         print*,' D312 ',dij(3,12),dij(3,12)/dijp(3,12)
         print*,' D313 ',dij(3,13),dij(3,13)/dijp(3,13)
      endif
cccccccccccc
      return
      end


c
c-------------  spence function  li2(z) ------------------------
c
      complex*16 function li2(zin)
      implicit none
      complex*16 zin, z, u, u2, unpo, ans, zext
      double precision r, r2, r2n, fac
c
c determine the value of the dilogarithm 
c
c    li2(z) = - int_0^1  log(1-zt)/t dt  with cut along the positive 
c                                        real axis, z>1
c
c	Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c	Initial version:  2000 November 6
c	Last modified:    2000 November 12
c  
      integer i
      double precision c0,c1,c2,c4,c6,c8,c10,c12,c14,c16,c18,c20,c22
      double precision b0,b1,b2,b4,b6,b8,b10,b12,b14,b16,b18,b20,b22
      double precision d0,d1,d2,d4,d6,d8,d10,d12,d14,d16,d18,d20,d22
      parameter (b0=1d0,            d0 =1d0,      c0= b0/d0)
      parameter (b1=-1d0/2d0,       d1 =d0*2d0,   c1= b1/d1)
      parameter (b2= 1d0/6d0,       d2 =d1*3d0,   c2= b2/d2)
      parameter (b4=-1d0/30d0,      d4 =d2*20d0,  c4= b4/d4)
      parameter (b6=1d0/42d0,       d6 =d4*42d0,  c6= b6/d6)
      parameter (b8=-1d0/30d0,      d8 =d6*72d0,  c8= b8/d8)
      parameter (b10=5d0/66d0,      d10=d8*110d0, c10=b10/d10)
      parameter (b12=-691d0/2730d0, d12=d10*156d0,c12=b12/d12)
      parameter (b14=7d0/6d0,       d14=d12*210d0,c14=b14/d14)
      parameter (b16=-3617d0/510d0, d16=d14*272d0,c16=b16/d16)
      parameter (b18=43867d0/798d0, d18=d16*342d0,c18=b18/d18)
      parameter (b20=-174611d0/330d0,d20=d18*420d0,c20=b20/d20)
      parameter (b22=854513d0/138d0,d22=d20*506d0,c22=b22/d22)
      double precision eps, epst, pi, pi2o6
      parameter (eps=1d-16, epst=1d-3)
      parameter (pi=3.14159 26535 89793, pi2o6=pi**2/6)
c
c debug information
      logical ldebug
      parameter (ldebug=.false.)

      z = zin
c      print*,' li2 call with z = ',z
      u = z**2
      r2 = dreal(z)**2+dimag(z)**2 
      if (r2.lt.eps) then
         li2 = z + u/4d0
         return
      elseif (r2.lt.epst) then
         ans = z + u/4
         do i = 3,11
            u = u*z
            ans = ans + u/i**2
         enddo
         li2 = ans
         return
      endif
      if (dreal(z).ge.1d0 .and. dimag(z).eq.0 ) then
         z = z + (0d0,1d0)*eps
      endif
c
c use z-->1/z and z--> 1-z mappings of the spence function to restrict 
c agument to unit circle in the complex plane with Re(z) <= 0.5
c
      zext = (0d0,0d0)
      fac = 1
      if (r2.gt.1d0) then     ! map z ---> 1/z
         fac = -fac
         zext = -pi2o6 - 0.5d0*(log(-z))**2
         z = 1/z
      endif
      if (dreal(z).gt.0.5d0) then     ! map new z ---> 1-z
         zext = zext + fac*(pi2o6-log(z)*log(1-z))
         fac = -fac
         z = 1-z
      endif
c
c now use t = 1 - exp(-u) mapping to write Li(z) in terms of Bernoulli 
c numbers
c
      u = - log(1-z)
      r2 = abs(u)**2
      u2 = u*u
      ans = u*(c0 + u*(c1+c2*u))
      r2n = r2*r2       !r^4

      unpo = u2*u2*u
      ans = ans + c4*unpo

      unpo = unpo*u2
      ans = ans + c6*unpo

      r = r2n*r2n       !r^8
      unpo = unpo*u2
      ans = ans + c8*unpo

      r2n = r*r2        !r^10 
      if ((r2n*c10).gt.eps) then
         unpo = unpo*u2
         ans = ans + c10*unpo
      else
         li2 = fac * ans + zext
         if (ldebug) print*,' exit li2s at n=8 '
         return
      endif

      unpo = unpo*u2
      ans = ans + c12*unpo

      unpo = unpo*u2
      ans = ans + c14*unpo

      unpo = unpo*u2
      ans = ans + c16*unpo

      r2n = r2n*r
      if ((r2n*c18).gt.eps) then
         unpo = unpo*u2
         ans = ans + c18*unpo
      else
         li2 = fac * ans + zext
         if (ldebug) print*,' exit li2s at n=16 '
         return
      endif

      unpo = unpo*u2
      ans = ans + c20*unpo

      unpo = unpo*u2
      ans = ans + c22*unpo

      li2 = fac * ans + zext
      end


c
c------------- C(m,q1^2,q2^2,P^2)  3-point fuction ---------------------------
c
      complex*16 function I3point(m,q1sq,q2sq,Psq)
      implicit none
      double precision m, q1sq, q2sq, psq 

c evaluate scalar 3-point function for equal masses m on propagators 
c  
c  I3 = 1/(i*pi^2) * Int d^4k [k^2-m^2]^-1 [(k+q1)^2-m^2]^-1 [(k-q2)^2-m^2]^-1
c
c     = i/pi^2 C('t Hooft,Veltman) = -C0(Passarino,Veltman)
c
c in terms of 12 dilogarithms. P = q1+q2 is assumed to be space-like. q1 
c and q2 may be space-, time- or lightlike
c
c	Dieter Zeppenfeld, <dieter@pheno.physics.wisc.edu>
c	Initial version:  2000 November 12
c	Last modified:    
c  

      double precision lambda, rtlam, q1q2, q1p, q2p, fmsq
      double precision rp(3), rm(3), eps, precise, pi

      complex*16 sp(3), sm(3), fmt2, ieps, fac, z, di, li2l, li2
      external li2, li2l
      parameter (eps=1d-18, ieps = eps*(0,1), precise=1d-3 )
      parameter (pi=3.14159 26535 89793, fac=-1d0/2d0)

c determine dot products of input momenta
      q1q2 = 0.5d0*(psq - q1sq - q2sq)
      q1p  = 0.5d0*(psq + q1sq - q2sq)
      q2p  = 0.5d0*(psq - q1sq + q2sq)
      lambda = q1q2**2-q1sq*q2sq
      
      
      if (lambda.le.0d0) then
         write(*,*) ' singular lambda in 3-point function. Reset to 0 '
         I3point = 0
         return
      else
         rtlam = sqrt(lambda)
      endif
      
      
      
c determine factors for call of spence functions
      fmsq = 4*m**2
      fmt2 = fmsq - ieps
      di = q1sq*q2sq*psq + fmt2*lambda 
      if ( abs(dreal(di)).gt.eps )then
         di = 1d0/di
      else
         print*,' singular point called in I3point '
         stop
      endif

      rp(1) = q2p + rtlam
      rm(1) = q2p - rtlam
      if (abs(rm(1)).lt.abs(rp(1))*precise ) then
         rm(1) = q2sq*psq/rp(1)
      elseif (abs(rp(1)).lt.abs(rm(1))*precise ) then
         rp(1) = q2sq*psq/rm(1)
      endif

      rp(2) = q1q2 + rtlam
      rm(2) = q1q2 - rtlam
      if (abs(rm(2)).lt.abs(rp(2))*precise ) then
         rm(2) = q1sq*q2sq/rp(2)
      elseif (abs(rp(2)).lt.abs(rm(2))*precise ) then
         rp(2) = q1sq*q2sq/rm(2)
      endif

      rp(3) = q1p + rtlam
      rm(3) = q1p - rtlam
      if (abs(rm(3)).lt.abs(rp(3))*precise ) then
         rm(3) = q1sq*psq/rp(3)
      elseif (abs(rp(3)).lt.abs(rm(3))*precise ) then
         rp(3) = q1sq*psq/rm(3)
      endif

      if (q1sq.ne.0d0) then
         z = rtlam*sqrt((q1sq-fmt2)*q1sq) 
         sp(1) = di*( q2p*q1sq + z )
         sm(1) = di*( q2p*q1sq - z )
      else
         sp(1) = 0
         sm(1) = 0
      endif

      if (psq.ne.0d0) then
         z = rtlam*sqrt((psq-fmt2)*psq) 
         sp(2) = di*( q1q2*psq + z )
         sm(2) = di*( q1q2*psq - z )
      else
         sp(2) = 0
         sm(2) = 0
      endif

      if (q2sq.ne.0d0) then
         z = rtlam*sqrt((q2sq-fmt2)*q2sq) 
         sp(3) = di*( q1p*q2sq + z )
         sm(3) = di*( q1p*q2sq - z )
      else
         sp(3) = 0
         sm(3) = 0
      endif

      z = 0
      if (q1sq.lt.0d0 .or. q1sq.gt.fmsq) then
         z = z + li2(rm(1)*sp(1)) + li2(rm(1)*sm(1)) - 
     &           li2(rp(1)*sm(1)) - li2(rp(1)*sp(1))
      elseif (q1sq.gt.0d0) then
         z = z + 2*dreal( li2(rm(1)*sp(1)) ) - 
     &           2*dreal( li2(rp(1)*sp(1)) )
      endif
      if (psq.lt.0d0 .or. psq.gt.fmsq) then
         z = z - li2(rm(2)*sp(2)) - li2(rm(2)*sm(2)) + 
     &           li2(rp(2)*sm(2)) + li2(rp(2)*sp(2))
      elseif (psq.gt.0d0) then
         z = z - 2*dreal( li2(rm(2)*sp(2)) ) + 
     &           2*dreal( li2(rp(2)*sp(2)) )
      endif
      if (q2sq.lt.0d0 .or. q2sq.gt.fmsq) then
         z = z + li2(rm(3)*sp(3)) + li2(rm(3)*sm(3)) - 
     &           li2(rp(3)*sm(3)) - li2(rp(3)*sp(3))
      elseif (q2sq.gt.0d0) then
         z = z + 2*dreal( li2(rm(3)*sp(3)) ) - 
     &           2*dreal( li2(rp(3)*sp(3)) )
      endif

      I3point = z*fac/rtlam
      return
      end
